@@ -1,4 +1,4 @@
-# u-msgpack-python v2.2 - v at sergeev.io
+# u-msgpack-python v2.3.0 - v at sergeev.io
 # https://github.com/vsergeev/u-msgpack-python
 #
 # u-msgpack-python is a lightweight MessagePack serializer and deserializer
@@ -10,7 +10,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2013-2016 Ivan (Vanya) A. Sergeev
+# Copyright (c) 2013-2016 vsergeev / Ivan (Vanya) A. Sergeev
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,7 @@
 # THE SOFTWARE.
 #
 """
-u-msgpack-python v2.2 - v at sergeev.io
+u-msgpack-python v2.3.0 - v at sergeev.io
 https://github.com/vsergeev/u-msgpack-python
 
 u-msgpack-python is a lightweight MessagePack serializer and deserializer
@@ -44,10 +44,10 @@ types.
 License: MIT
 """
 
-__version__ = "2.2"
+__version__ = "2.3.0"
 "Module version string"
 
-version = (2,2)
+version = (2,3,0)
 "Module version tuple"
 
 import struct
@@ -117,18 +117,16 @@ class Ext:
         String representation of this Ext object.
         """
         s = "Ext Object (Type: 0x%02x, Data: " % self.type
-        for i in range(min(len(self.data), 8)):
-            if i > 0:
-                s += " "
-            if isinstance(self.data[i], int):
-                s += "%02x" % (self.data[i])
-            else:
-                s += "%02x" % ord(self.data[i])
+        s += " ".join(["0x%02x" % ord(self.data[i:i+1]) for i in xrange(min(len(self.data), 8))])
         if len(self.data) > 8:
             s += " ..."
         s += ")"
         return s
 
+class InvalidString(bytes):
+    """Subclass of bytes to hold invalid UTF-8 strings."""
+    pass
+
 ################################################################################
 ### Exceptions
 ################################################################################
@@ -148,7 +146,7 @@ class UnsupportedTypeException(PackException):
 
 # Unpacking error
 class InsufficientDataException(UnpackException):
-    "Insufficient data to unpack the encoded object."
+    "Insufficient data to unpack the serialized object."
     pass
 class InvalidStringException(UnpackException):
     "Invalid UTF-8 string encountered during unpacking."
@@ -212,7 +210,7 @@ b'\x92\xabsome string\xaasome bytes'
 # chr(obj) has a str return type instead of bytes in Python 3, and
 # struct.pack(...) has the right return type in both versions.
 
-def _pack_integer(obj, fp):
+def _pack_integer(obj, fp, options):
     if obj < 0:
         if obj >= -32:
             fp.write(struct.pack("b", obj))
@@ -240,19 +238,19 @@ def _pack_integer(obj, fp):
         else:
             raise UnsupportedTypeException("huge unsigned int")
 
-def _pack_nil(obj, fp):
+def _pack_nil(obj, fp, options):
     fp.write(b"\xc0")
 
-def _pack_boolean(obj, fp):
+def _pack_boolean(obj, fp, options):
     fp.write(b"\xc3" if obj else b"\xc2")
 
-def _pack_float(obj, fp):
+def _pack_float(obj, fp, options):
     if _float_size == 64:
         fp.write(b"\xcb" + struct.pack(">d", obj))
     else:
         fp.write(b"\xca" + struct.pack(">f", obj))
 
-def _pack_string(obj, fp):
+def _pack_string(obj, fp, options):
     obj = obj.encode('utf-8')
     if len(obj) <= 31:
         fp.write(struct.pack("B", 0xa0 | len(obj)) + obj)
@@ -265,7 +263,7 @@ def _pack_string(obj, fp):
     else:
         raise UnsupportedTypeException("huge string")
 
-def _pack_binary(obj, fp):
+def _pack_binary(obj, fp, options):
     if len(obj) <= 2**8-1:
         fp.write(b"\xc4" + struct.pack("B", len(obj)) + obj)
     elif len(obj) <= 2**16-1:
@@ -275,7 +273,7 @@ def _pack_binary(obj, fp):
     else:
         raise UnsupportedTypeException("huge binary string")
 
-def _pack_oldspec_raw(obj, fp):
+def _pack_oldspec_raw(obj, fp, options):
     if len(obj) <= 31:
         fp.write(struct.pack("B", 0xa0 | len(obj)) + obj)
     elif len(obj) <= 2**16-1:
@@ -285,7 +283,7 @@ def _pack_oldspec_raw(obj, fp):
     else:
         raise UnsupportedTypeException("huge raw string")
 
-def _pack_ext(obj, fp):
+def _pack_ext(obj, fp, options):
     if len(obj.data) == 1:
         fp.write(b"\xd4" + struct.pack("B", obj.type & 0xff) + obj.data)
     elif len(obj.data) == 2:
@@ -305,7 +303,7 @@ def _pack_ext(obj, fp):
     else:
         raise UnsupportedTypeException("huge ext data")
 
-def _pack_array(obj, fp):
+def _pack_array(obj, fp, options):
     if len(obj) <= 15:
         fp.write(struct.pack("B", 0x90 | len(obj)))
     elif len(obj) <= 2**16-1:
@@ -316,9 +314,9 @@ def _pack_array(obj, fp):
         raise UnsupportedTypeException("huge array")
 
     for e in obj:
-        pack(e, fp)
+        pack(e, fp, **options)
 
-def _pack_map(obj, fp):
+def _pack_map(obj, fp, options):
     if len(obj) <= 15:
         fp.write(struct.pack("B", 0x80 | len(obj)))
     elif len(obj) <= 2**16-1:
@@ -329,13 +327,13 @@ def _pack_map(obj, fp):
         raise UnsupportedTypeException("huge array")
 
     for k,v in obj.items():
-        pack(k, fp)
-        pack(v, fp)
+        pack(k, fp, **options)
+        pack(v, fp, **options)
 
 ########################################
 
 # Pack for Python 2, with 'unicode' type, 'str' type, and 'long' type
-def _pack2(obj, fp):
+def _pack2(obj, fp, **options):
     """
     Serialize a Python object into MessagePack bytes.
 
@@ -343,6 +341,11 @@ def _pack2(obj, fp):
         obj: a Python object
         fp: a .write()-supporting file-like object
 
+    Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping a custom type
+                             to a callable that packs an instance of the type
+                             into an Ext object
+
     Returns:
         None.
 
@@ -355,36 +358,46 @@ def _pack2(obj, fp):
     >>> umsgpack.pack({u"compact": True, u"schema": 0}, f)
     >>>
     """
-
     global compatibility
 
+    ext_handlers = options.get("ext_handlers")
+
     if obj is None:
-        _pack_nil(obj, fp)
+        _pack_nil(obj, fp, options)
+    elif ext_handlers and obj.__class__ in ext_handlers:
+        _pack_ext(ext_handlers[obj.__class__](obj), fp, options)
     elif isinstance(obj, bool):
-        _pack_boolean(obj, fp)
+        _pack_boolean(obj, fp, options)
     elif isinstance(obj, int) or isinstance(obj, long):
-        _pack_integer(obj, fp)
+        _pack_integer(obj, fp, options)
     elif isinstance(obj, float):
-        _pack_float(obj, fp)
+        _pack_float(obj, fp, options)
     elif compatibility and isinstance(obj, unicode):
-        _pack_oldspec_raw(bytes(obj), fp)
+        _pack_oldspec_raw(bytes(obj), fp, options)
     elif compatibility and isinstance(obj, bytes):
-        _pack_oldspec_raw(obj, fp)
+        _pack_oldspec_raw(obj, fp, options)
     elif isinstance(obj, unicode):
-        _pack_string(obj, fp)
+        _pack_string(obj, fp, options)
     elif isinstance(obj, str):
-        _pack_binary(obj, fp)
+        _pack_binary(obj, fp, options)
     elif isinstance(obj, list) or isinstance(obj, tuple):
-        _pack_array(obj, fp)
+        _pack_array(obj, fp, options)
     elif isinstance(obj, dict):
-        _pack_map(obj, fp)
+        _pack_map(obj, fp, options)
     elif isinstance(obj, Ext):
-        _pack_ext(obj, fp)
+        _pack_ext(obj, fp, options)
+    elif ext_handlers:
+        # Linear search for superclass
+        t = next((t for t in ext_handlers.keys() if isinstance(obj, t)), None)
+        if t:
+            _pack_ext(ext_handlers[t](obj), fp, options)
+        else:
+            raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
     else:
         raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
 
 # Pack for Python 3, with unicode 'str' type, 'bytes' type, and no 'long' type
-def _pack3(obj, fp):
+def _pack3(obj, fp, **options):
     """
     Serialize a Python object into MessagePack bytes.
 
@@ -392,6 +405,11 @@ def _pack3(obj, fp):
         obj: a Python object
         fp: a .write()-supporting file-like object
 
+    Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping a custom type
+                             to a callable that packs an instance of the type
+                             into an Ext object
+
     Returns:
         None.
 
@@ -406,38 +424,54 @@ def _pack3(obj, fp):
     """
     global compatibility
 
+    ext_handlers = options.get("ext_handlers")
+
     if obj is None:
-        _pack_nil(obj, fp)
+        _pack_nil(obj, fp, options)
+    elif ext_handlers and obj.__class__ in ext_handlers:
+        _pack_ext(ext_handlers[obj.__class__](obj), fp, options)
     elif isinstance(obj, bool):
-        _pack_boolean(obj, fp)
+        _pack_boolean(obj, fp, options)
     elif isinstance(obj, int):
-        _pack_integer(obj, fp)
+        _pack_integer(obj, fp, options)
     elif isinstance(obj, float):
-        _pack_float(obj, fp)
+        _pack_float(obj, fp, options)
     elif compatibility and isinstance(obj, str):
-        _pack_oldspec_raw(obj.encode('utf-8'), fp)
+        _pack_oldspec_raw(obj.encode('utf-8'), fp, options)
     elif compatibility and isinstance(obj, bytes):
-        _pack_oldspec_raw(obj, fp)
+        _pack_oldspec_raw(obj, fp, options)
     elif isinstance(obj, str):
-        _pack_string(obj, fp)
+        _pack_string(obj, fp, options)
     elif isinstance(obj, bytes):
-        _pack_binary(obj, fp)
+        _pack_binary(obj, fp, options)
     elif isinstance(obj, list) or isinstance(obj, tuple):
-        _pack_array(obj, fp)
+        _pack_array(obj, fp, options)
     elif isinstance(obj, dict):
-        _pack_map(obj, fp)
+        _pack_map(obj, fp, options)
     elif isinstance(obj, Ext):
-        _pack_ext(obj, fp)
+        _pack_ext(obj, fp, options)
+    elif ext_handlers:
+        # Linear search for superclass
+        t = next((t for t in ext_handlers.keys() if isinstance(obj, t)), None)
+        if t:
+            _pack_ext(ext_handlers[t](obj), fp, options)
+        else:
+            raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
     else:
         raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
 
-def _packb2(obj):
+def _packb2(obj, **options):
     """
     Serialize a Python object into MessagePack bytes.
 
     Args:
         obj: a Python object
 
+    Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping a custom type
+                             to a callable that packs an instance of the type
+                             into an Ext object
+
     Returns:
         A 'str' containing serialized MessagePack bytes.
 
@@ -451,16 +485,21 @@ def _packb2(obj):
     >>>
     """
     fp = io.BytesIO()
-    _pack2(obj, fp)
+    _pack2(obj, fp, **options)
     return fp.getvalue()
 
-def _packb3(obj):
+def _packb3(obj, **options):
     """
     Serialize a Python object into MessagePack bytes.
 
     Args:
         obj: a Python object
 
+    Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping a custom type
+                             to a callable that packs an instance of the type
+                             into an Ext object
+
     Returns:
         A 'bytes' containing serialized MessagePack bytes.
 
@@ -474,7 +513,7 @@ def _packb3(obj):
     >>>
     """
     fp = io.BytesIO()
-    _pack3(obj, fp)
+    _pack3(obj, fp, **options)
     return fp.getvalue()
 
 ################################################################################
@@ -551,10 +590,13 @@ def _unpack_string(code, fp, options):
     if compatibility:
         return _read_except(fp, length)
 
+    data = _read_except(fp, length)
     try:
-        return bytes.decode(_read_except(fp, length), 'utf-8')
+        return bytes.decode(data, 'utf-8')
     except UnicodeDecodeError:
-        raise InvalidStringException("unpacked string is not utf-8")
+        if options.get("allow_invalid_utf8"):
+            return InvalidString(data)
+        raise InvalidStringException("unpacked string is invalid utf-8")
 
 def _unpack_binary(code, fp, options):
     if code == b'\xc4':
@@ -588,7 +630,14 @@ def _unpack_ext(code, fp, options):
     else:
         raise Exception("logic error, not ext: 0x%02x" % ord(code))
 
-    return Ext(ord(_read_except(fp, 1)), _read_except(fp, length))
+    ext = Ext(ord(_read_except(fp, 1)), _read_except(fp, length))
+
+    # Unpack with ext handler, if we have one
+    ext_handlers = options.get("ext_handlers")
+    if ext_handlers and ext.type in ext_handlers:
+        ext = ext_handlers[ext.type](ext)
+
+    return ext
 
 def _unpack_array(code, fp, options):
     if (ord(code) & 0xf0) == 0x90:
@@ -653,15 +702,21 @@ def _unpack2(fp, **options):
         fp: a .read()-supporting file-like object
 
     Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping integer Ext
+                             type to a callable that unpacks an instance of
+                             Ext into an object
         use_ordered_dict (bool): unpack maps into OrderedDict, instead of
                                  unordered dict (default False)
+        allow_invalid_utf8 (bool): unpack invalid strings into instances of
+                                   InvalidString, for access to the bytes
+                                   (default False)
 
     Returns:
         A Python object.
 
     Raises:
         InsufficientDataException(UnpackException):
-            Insufficient data to unpack the encoded object.
+            Insufficient data to unpack the serialized object.
         InvalidStringException(UnpackException):
             Invalid UTF-8 string encountered during unpacking.
         ReservedCodeException(UnpackException):
@@ -688,15 +743,21 @@ def _unpack3(fp, **options):
         fp: a .read()-supporting file-like object
 
     Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping integer Ext
+                             type to a callable that unpacks an instance of
+                             Ext into an object
         use_ordered_dict (bool): unpack maps into OrderedDict, instead of
                                  unordered dict (default False)
+        allow_invalid_utf8 (bool): unpack invalid strings into instances of
+                                   InvalidString, for access to the bytes
+                                   (default False)
 
     Returns:
         A Python object.
 
     Raises:
         InsufficientDataException(UnpackException):
-            Insufficient data to unpack the encoded object.
+            Insufficient data to unpack the serialized object.
         InvalidStringException(UnpackException):
             Invalid UTF-8 string encountered during unpacking.
         ReservedCodeException(UnpackException):
@@ -724,8 +785,14 @@ def _unpackb2(s, **options):
         s: a 'str' or 'bytearray' containing serialized MessagePack bytes
 
     Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping integer Ext
+                             type to a callable that unpacks an instance of
+                             Ext into an object
         use_ordered_dict (bool): unpack maps into OrderedDict, instead of
                                  unordered dict (default False)
+        allow_invalid_utf8 (bool): unpack invalid strings into instances of
+                                   InvalidString, for access to the bytes
+                                   (default False)
 
     Returns:
         A Python object.
@@ -734,7 +801,7 @@ def _unpackb2(s, **options):
         TypeError:
             Packed data type is neither 'str' nor 'bytearray'.
         InsufficientDataException(UnpackException):
-            Insufficient data to unpack the encoded object.
+            Insufficient data to unpack the serialized object.
         InvalidStringException(UnpackException):
             Invalid UTF-8 string encountered during unpacking.
         ReservedCodeException(UnpackException):
@@ -763,8 +830,15 @@ def _unpackb3(s, **options):
         s: a 'bytes' or 'bytearray' containing serialized MessagePack bytes
 
     Kwargs:
+        ext_handlers (dict): dictionary of Ext handlers, mapping integer Ext
+                             type to a callable that unpacks an instance of
+                             Ext into an object
         use_ordered_dict (bool): unpack maps into OrderedDict, instead of
                                  unordered dict (default False)
+        allow_invalid_utf8 (bool): unpack invalid strings into instances of
+                                   InvalidString, for access to the bytes
+                                   (default False)
+
     Returns:
         A Python object.
 
@@ -772,7 +846,7 @@ def _unpackb3(s, **options):
         TypeError:
             Packed data type is neither 'bytes' nor 'bytearray'.
         InsufficientDataException(UnpackException):
-            Insufficient data to unpack the encoded object.
+            Insufficient data to unpack the serialized object.
         InvalidStringException(UnpackException):
             Invalid UTF-8 string encountered during unpacking.
         ReservedCodeException(UnpackException):
